package com.mediahub

import akka.NotUsed
import akka.actor.ActorSystem
import akka.stream.ActorAttributes.supervisionStrategy
import akka.stream.Supervision.resumingDecider
import akka.stream.alpakka.csv.scaladsl.{CsvParsing, CsvToMap}
import akka.stream.scaladsl.{Compression, FileIO, Flow, Sink, Source}
import akka.util.ByteString
import org.springframework.stereotype.Component

import java.io.File
import java.nio.file.Paths
import scala.collection.immutable
import scala.concurrent.duration.DurationInt
import scala.concurrent.{Await, Future}
import scala.language.postfixOps


@Component
class FileReader {

  implicit val system: ActorSystem = ActorSystem()

  val titleBasicsResource: File = new File("src/main/resources/title.basics.tsv.gz")
  val titlePrincipalsResource: File = new File("src/main/resources/title.principals.tsv.gz")
  val nameBasicsResource: File = new File("src/main/resources/name.basics.tsv.gz")

  def fileSource(file: File): Source[ByteString, NotUsed] = {
    Source.single(file)
      .flatMapConcat(f => FileIO.fromPath(Paths.get(f.getPath), 1 * 1024 * 1024 * 2))
      .via(Compression.gunzip())
  }

  val lineParser: Flow[ByteString, Map[String, String], NotUsed] = {
    CsvParsing
      .lineScanner(CsvParsing.Tab, CsvParsing.DoubleQuote, CsvParsing.DoubleQuote)
      .via(CsvToMap.toMapAsStringsCombineAll(headerPlaceholder = Option.empty))
  }

  //working search not optimized
  val titleId: String = getIdOfTitleAsync(titleBasicsResource, "Carmencita")
  println("value ", titleId)

  val personsIdList: List[String] = getIdOfPersonsAsync(titlePrincipalsResource, titleId).toList.flatten
  println("value ", personsIdList)

  val personsList = getPersons(nameBasicsResource, personsIdList)
  println("value ", personsList)


  def getIdOfTitleAsync(file: File, titleName: String): String = {
    val result = fileSource(file)
      .mapAsync(50) {
        res =>
          Source.single(res)
            .via(lineParser)
            .filter(row => row.getOrElse("primaryTitle", "") == titleName)
            .map(a => a.get("tconst"))
            .withAttributes(supervisionStrategy(resumingDecider))
            .runWith(Sink.head)
      }
      .withAttributes(supervisionStrategy(resumingDecider))
      .runWith(Sink.head)

    Await.result(result, 5 minutes)
    result.value.get.get.get
  }

  def getIdOfPersonsAsync(file: File, titleId: String): immutable.Iterable[Option[String]] = {
    val te = fileSource(file)
      .mapAsync(50) {
        res =>
          Source.single(res)
            .via(lineParser)
            .filter(row => row.getOrElse("tconst", "") == titleId)
            .map(a => a.get("nconst"))
            .withAttributes(supervisionStrategy(resumingDecider))
            .runWith(Sink.collection)
      }
      .withAttributes(supervisionStrategy(resumingDecider))
      .runWith(Sink.head)

    Await.result(te, 5 minutes)
  }

  def getPersons(file: File, personsIdList: List[String]): List[String] = {
    val result: Future[immutable.Iterable[Option[String]]] = Source.single(file)
      .flatMapConcat(f => FileIO.fromPath(Paths.get(f.getPath), 1 * 1024 * 1024))
      .via(Compression.gunzip())
      .via(CsvParsing.lineScanner(CsvParsing.Tab, CsvParsing.DoubleQuote, CsvParsing.DoubleQuote))
      .via(CsvToMap.toMapAsStringsCombineAll(headerPlaceholder = Option.empty))
      .filter(row => personsIdList.contains(row.getOrElse("nconst", "")))
      .map(a => a.get("primaryName"))
      .runWith(Sink.collection)

    Await.result(result, 5 minutes)
    result.value.get.get.toList.flatten
  }

 /* def getPersonsAsync(file: File, personsIdList: List[String]) = {
    val test = Source.single(file)
      .flatMapConcat(f => FileIO.fromPath(Paths.get(f.getPath), 1 * 1024 * 1024))
      .via(Compression.gunzip())
      .mapAsync(2) {
        res =>
          Source.single(res)
            .via(CsvParsing.lineScanner(CsvParsing.Tab, CsvParsing.DoubleQuote, CsvParsing.DoubleQuote))
            .via(CsvToMap.toMapAsStringsCombineAll(headerPlaceholder = Option.empty))
            .filter(row => row.getOrElse("nconst", "") == personsIdList.head)
            .map(a => a.get("primaryName"))
            .runWith(Sink.collection)
      }
      .runWith(Sink.collection)

    Await.result(test, 5 minutes)
    test.value.get.get.toList.flatten
  }
  */
}